from .entry import Entry
from typing import Optional, List


class Feed(object):
    page: int
    name: str
    entries: List['Entry']

    def __init__(self, page: int, name: str, entries: List['Entry']):
        self.page = page
        self.name = name
        self.entries = entries



import requests

from .entry import Entry
from .post import Post
from .feed import Feed


class HNApi(object):
    def __init__(self, *args, **kwargs):
        self.s = requests.Session()
        self.feed_limits = {
            "news": 10,
            "newest": 12,
            "ask": 2,
            "show": 2,
            "jobs": 1
        }

    def feed(self, feed_type, pagination=1):
        if pagination < self.feed_limits[feed_type]:
            feed_entries = self.s.get(f'https://api.hnpwa.com/v0/{feed_type}/{pagination}.json').json()
        else:
            feed_entries = self.s.get(f'https://api.hnpwa.com/v0/{feed_type}/{self.feed_limits[feed_type]}.json').json()
        entries = [Entry(**feed_entry, **{'index': ((pagination - 1) * 30) + i}) for i, feed_entry in enumerate(feed_entries, start=1)]
        return Feed(pagination, feed_type, entries)


    def post_item(self, post_id):
        post = self.s.get(f'https://api.hnpwa.com/v0/item/{post_id}.json').json()
        return Post(**post)



from enum import Enum
from typing import Optional


class TypeEnum(Enum):
    JOB = "job"
    LINK = "link"
    ASK = "ask"


class Entry:
    comments_count: int
    domain: Optional[str]
    id: int
    index: Optional[int]
    points: Optional[int]
    time: int
    time_ago: str
    title: str
    type: TypeEnum
    url: str
    user: Optional[str]

    def __init__(self, comments_count: int, id: int, points: Optional[int], time: int, time_ago: str, title: str, index: Optional[int], type: TypeEnum, url: str, user: Optional[str], domain: str = None) -> None:
        self.comments_count = comments_count
        self.domain = domain
        self.id = id
        self.points = points
        self.time = time
        self.time_ago = time_ago
        self.title = title
        self.type = type
        self.url = url
        self.user = user
        self.index = index

    def __repr__(self):
        values = "\n".join([f"\t'{k}': {repr(v)}" for k,v in self.__dict__.items()])
        return f"{self.__class__} ({{\n{values}\n}})"


import json
import copy
from enum import Enum
from typing import Optional, List


class TypeEnum(Enum):
    COMMENT = "comment"


class Comment:
    id: int
    user: Optional[str]
    time: int
    time_ago: str
    type: TypeEnum
    content: str
    comments: List['Comment']
    comments_count: int
    level: int
    url: str
    # deleted: Optional[bool]
    # dead: Optional[bool]

    def __init__(self, id: int, time: int, time_ago: str, type: TypeEnum, content: str, comments: List['Comment'], comments_count: int, level: int, url: str, user: Optional[str] = None, deleted: Optional[bool] = None, dead: Optional[bool] = None) -> None:
        self.id = id
        self.user = user
        self.time = time
        self.time_ago = time_ago
        self.type = type
        self.content = content
        self.comments = [Comment(**c) for c in comments]
        self.comments_count = comments_count
        self.level = level
        self.url = url

    def __repr__(self):
        self_dict = copy.copy(self.__dict__)
        self_dict.update({'comments': [str(c.__class__) for c in self_dict.get('comments')]})
        values = json.dumps(self_dict, indent=4)
        return f"{self.__class__} ({values})"


class Post:
    id: int
    title: str
    points: int
    user: Optional[str]
    domain: Optional[str]
    time: int
    time_ago: str
    type: str
    content: str
    comments: List[Comment]
    comments_count: int
    url: str

    def __init__(self, id: int, title: str, points: int, user: Optional[str], time: int, time_ago: str, type: str, content: str, comments: List[Comment], comments_count: int, url: str, domain: Optional[str] = None) -> None:
        self.id = id
        self.title = title
        self.points = points
        self.user = user
        self.time = time
        self.time_ago = time_ago
        self.type = type
        self.content = content
        self.domain = domain
        self.comments = [Comment(**c) for c in comments]
        self.comments_count = comments_count
        self.url = url

    def __repr__(self):
        self_dict = copy.copy(self.__dict__)
        self_dict.update({'comments': [str(c.__class__) for c in self_dict.get('comments')]})
        values = json.dumps(self_dict, indent=4)
        return f"{self.__class__} ({values})"